package dk.sosi.sts.nemlogin.uuid2cpr;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.junit.Before;
import org.junit.Test;

import javax.xml.ws.soap.SOAPFaultException;
import java.util.Optional;

import static org.junit.Assert.*;

/**
 * The tests lookupCprFromUuid_*  tests correspond to section 4 of https://jira.nspop.dk/secure/attachment/13536/13536_Integration+med+UUID2CPR+opslagsservice+v01.pdf
 * Tests regarding token caching and expiry have been deliberately omitted - they are cumbersome to write, and would essentially just test that
 * the CXF framework handles token caching correctly (which we assume it does).
 */
public class Uuid2CprClientIT {
    private Uuid2CprClient subject;

    @Before
    public void setup() {
        Uuid2CprProperties properties = new Uuid2CprProperties();
        properties.setAppliesTo("https://saml.wsp.lookupservice.devtest4-nemlog-in.dk");
        properties.setSignatureUsername("lookup");
        properties.setSignaturePassword("Test1234");
        properties.setSignaturePropertiesFile("client.properties");
        properties.setEncryptionUsername("sts");
        properties.setEncryptionPropertiesFile("sts.properties");

        properties.setLookupServiceUrl("https://lookupservice.devtest4-nemlog-in.dk/LookupService.svc");
        properties.setSecurityTokenServiceUrl("https://SecureTokenService.devtest4-nemlog-in.dk/SecurityTokenService.svc");

        subject = new Uuid2CprClient(new GenericObjectPool<>(new LookupServiceObjectFactory(properties)));
    }
    
 // uuid2cpr *******************************************************************************************************
    
    @Test
    public void lookupCprFromUuid_globalUuid_success() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";

        // Act
        Optional<String> cpr = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        assertTrue(cpr.isPresent());
        assertEquals("2501879875", cpr.get());
    }

    @Test
    public void lookupCprFromUuid_globalUuid_uuidUnknown_failure() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46eae";

        // Act
        Optional<String> cpr = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        assertFalse(cpr.isPresent());
    }

    @Test
    public void lookupCprFromUuid_certificateSpecificUuid_success() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:edc04d69-f2d4-4e63-b9ed-593f06c1ff5d";

        // Act
        Optional<String> cpr = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        assertTrue(cpr.isPresent());
        assertEquals("2501879875", cpr.get());
    }

    @Test
    public void lookupCprFromUuid_certificateSpecificUuid_uuidUnknown_failure() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:edc04d69-f2d4-4e63-b9ed-593f06c1ff5e";

        // Act
        Optional<String> cpr = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        assertFalse(cpr.isPresent());
    }

    @Test(expected = SOAPFaultException.class)
    public void lookupCprFromUuid_malformedSubjectSerialNumber_failure() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:not-even-remotely-correct-input";

        // Act
        Optional<String> cpr = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
    }
 
    // pid2cpr *******************************************************************************************************
    
    @Test
    public void lookupCprFromPid_success() throws Exception {
        // Arrange
        String pid = "9208-2002-2-199567135534";

        // Act
        Optional<String> cpr = subject.lookupCprFromPid(pid);

        // Assert
        assertTrue(cpr.isPresent());
        assertEquals("2501879875", cpr.get());
    }
    
    @Test
    public void lookupCprFromPid_pidUnknown_failure() throws Exception {
        // Arrange
    	String pid = "9208-2002-2-1995671355340001";

        // Act
    	Optional<String> cpr = subject.lookupCprFromPid(pid);

        // Assert
        assertFalse(cpr.isPresent());
    }

    // uuid2rid *******************************************************************************************************
    
    @Test
    public void lookupRidFromUuid_globalUuid_success() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";

        // Act
        Optional<String> rid = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        assertTrue(rid.isPresent());
        assertEquals("72976780", rid.get());
    }
    
    @Test
    public void lookupRidFromUuid_globalUuid_uuidUnknown_failure() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46eae";

        // Act
        Optional<String> rid = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        assertFalse(rid.isPresent());
    }

    @Test
    public void lookupRidFromUuidd_certificateSpecificUuid_success() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:edc04d69-f2d4-4e63-b9ed-593f06c1ff5d";

        // Act
        Optional<String> rid = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        assertTrue(rid.isPresent());
        assertEquals("72976780", rid.get());
    }

    @Test
    public void lookupRidFromUuid_certificateSpecificUuid_uuidUnknown_failure() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:edc04d69-f2d4-4e63-b9ed-593f06c1ff5e";

        // Act
        Optional<String> rid = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        assertFalse(rid.isPresent());
    }

    @Test(expected = SOAPFaultException.class)
    public void lookupRidFromUuid_malformedSubjectSerialNumber_failure() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:C:not-even-remotely-correct-input";

        // Act
        Optional<String> rid = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
    }
    
}