package dk.sosi.sts.nemlogin.uuid2cpr;

import dk.nemlog_in.lookupservice.LookupService;
import org.apache.commons.pool.ObjectPool;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.CprByPidResponse;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.CprBySubjectSerialNumberResponse;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.RidBySubjectSerialNumberResponse;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import java.util.Optional;

import static org.junit.Assert.*;

public class Uuid2CprClientTest {
    private Uuid2CprClient subject;

    private ObjectPool<LookupService> clientPool;

    private LookupService client;

    @Before
    public void setup() throws Exception {
        client = Mockito.mock(LookupService.class);

        clientPool = Mockito.mock(ObjectPool.class);
        Mockito.when(clientPool.borrowObject()).thenReturn(client);

        subject = new Uuid2CprClient(clientPool);
    }
    
    // uuid2cpr *******************************************************************************************************
    
    @Test
    public void lookupCprFromUuid_cprPresent_returnsCpr() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";
        String cpr = "1802602810";

        Mockito.when(client.subjectSerialNumberCpr(subjectSerialNumber)).thenReturn(buildCprBySubjectSerialNumberResponse(cpr));

        // Act
        Optional<String> result = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        assertTrue(result.isPresent());
        assertEquals(cpr, result.get());
    }

    @Test
    public void lookupCprFromUuid_cprMissing_returnsEmptyResult() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";

        Mockito.when(client.subjectSerialNumberCpr(subjectSerialNumber)).thenReturn(buildCprBySubjectSerialNumberResponse(null));

        // Act
        Optional<String> result = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        assertFalse(result.isPresent());
    }

    @Test
    public void lookupCprFromUuid_borrowsObjectIsReturned() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";
        String cpr = "1802602810";

        Mockito.when(client.subjectSerialNumberCpr(subjectSerialNumber)).thenReturn(buildCprBySubjectSerialNumberResponse(cpr));

        // Act
        Optional<String> result = subject.lookupCprFromUuid(subjectSerialNumber);

        // Assert
        Mockito.verify(clientPool).returnObject(client);
    }
    
    private CprBySubjectSerialNumberResponse buildCprBySubjectSerialNumberResponse(String cpr) {
        CprBySubjectSerialNumberResponse response = new CprBySubjectSerialNumberResponse();
        response.setCpr(new JAXBElement<>(new QName("http://foo", "bar"), String.class, cpr));
        return response;
    }
    
    // pid2cpr *******************************************************************************************************
    
    @Test
    public void lookupCprFromPid_cprPresent_returnsCpr() throws Exception {
        // Arrange
        String pid = "9208-2002-2-130462414956";
        String cpr = "1802602810";

        Mockito.when(client.pidCpr(pid)).thenReturn(buildCprByPidResponse(cpr));

        // Act
        Optional<String> result = subject.lookupCprFromPid(pid);

        // Assert
        assertTrue(result.isPresent());
        assertEquals(cpr, result.get());
    }

    @Test
    public void lookupCprFromPid_cprMissing_returnsEmptyResult() throws Exception {
        // Arrange
        String pid = "9208-2002-2-130462414956";

        Mockito.when(client.pidCpr(pid)).thenReturn(buildCprByPidResponse(null));

        // Act
        Optional<String> result = subject.lookupCprFromPid(pid);

        // Assert
        assertFalse(result.isPresent());
    }

    @Test
    public void lookupCprFromPid_borrowsObjectIsReturned() throws Exception {
        // Arrange
        String pid = "9208-2002-2-130462414956";
        String cpr = "1802602810";

        Mockito.when(client.pidCpr(pid)).thenReturn(buildCprByPidResponse(cpr));

        // Act
        Optional<String> result = subject.lookupCprFromPid(pid);

        // Assert
        Mockito.verify(clientPool).returnObject(client);
    }
    
    private CprByPidResponse buildCprByPidResponse(String cpr) {
    	CprByPidResponse response = new CprByPidResponse();
        response.setCpr(new JAXBElement<>(new QName("http://foo", "bar"), String.class, cpr));
        return response;
    }
    
    // uuid2rid *******************************************************************************************************
    
    @Test
    public void lookupRidFromUuid_cprPresent_returnsCpr() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";
        String rid = "19822376";

        Mockito.when(client.subjectSerialNumberRid(subjectSerialNumber)).thenReturn(buildRidBySubjectSerialNumberResponse(rid));

        // Act
        Optional<String> result = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        assertTrue(result.isPresent());
        assertEquals(rid, result.get());
    }

    @Test
    public void lookupRidFromUuid_cprMissing_returnsEmptyResult() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";

        Mockito.when(client.subjectSerialNumberRid(subjectSerialNumber)).thenReturn(buildRidBySubjectSerialNumberResponse(null));

        // Act
        Optional<String> result = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        assertFalse(result.isPresent());
    }

    @Test
    public void lookupRidFromUuid_borrowsObjectIsReturned() throws Exception {
        // Arrange
        String subjectSerialNumber = "UI:DK-E:G:2634ccc3-225a-44ee-94bc-565904f46ead";
        String rid = "19822376";

        Mockito.when(client.subjectSerialNumberRid(subjectSerialNumber)).thenReturn(buildRidBySubjectSerialNumberResponse(rid));

        // Act
        Optional<String> result = subject.lookupRidFromUuid(subjectSerialNumber);

        // Assert
        Mockito.verify(clientPool).returnObject(client);
    }

    private RidBySubjectSerialNumberResponse buildRidBySubjectSerialNumberResponse(String rid) {
    	RidBySubjectSerialNumberResponse response = new RidBySubjectSerialNumberResponse();
        response.setRid(new JAXBElement<>(new QName("http://foo", "bar"), String.class, rid));
        return response;
    }
}