/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2018 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package dk.nsp.uuid2cpr.proxy.setup;

import dk.nemlog_in.lookupservice.LookupService;
import dk.nsp.uuid2cpr.proxy.health.CertificateHealthChecker;
import dk.nsp.uuid2cpr.proxy.health.CompositeHealthChecker;
import dk.nsp.uuid2cpr.proxy.health.HealthChecker;
import dk.nsp.uuid2cpr.proxy.ws.ServiceApiImpl;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriter;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriterImpl;
import dk.sdsd.nsp.slalog.api.SLALogConfig;
import dk.sdsd.nsp.slalog.api.SLALogger;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.FileBasedCredentialVault;
import dk.sosi.sts.nemlogin.uuid2cpr.LookupServiceObjectFactory;
import dk.sosi.sts.nemlogin.uuid2cpr.Uuid2CprClient;
import dk.sosi.sts.nemlogin.uuid2cpr.Uuid2CprProperties;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.PoolableObjectFactory;
import org.apache.commons.pool.impl.GenericObjectPool;
import org.openapitools.api.ServiceApi;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;

import java.io.File;
import java.util.Arrays;
import java.util.Properties;

@Configuration
@PropertySource("classpath:uuid2cpr-proxy.properties")
@PropertySource("classpath:git.properties")
public class Uuid2CprProxySetup {
	@Bean
	public ServiceApi serviceApi(@Autowired SLALogger slaLogger, @Autowired Uuid2CprClient client, @Value("${uuid2cpr.proxy.lookup.service.url}") String lookupServiceUrl) {
		return new ServiceApiImpl(slaLogger, client, lookupServiceUrl);
	}

	@Bean
	public OpenApiWriter openApiWriter(@Value("${app.endpoint}") String endpoint) {
		return new OpenApiWriterImpl(endpoint);
	}

    @Bean
    public HealthChecker healthChecker(@Autowired @Qualifier("keystoreHealthChecker") HealthChecker keystoreHealthChecker, @Autowired @Qualifier("truststoreHealthChecker") HealthChecker truststoreHealthChecker) {
        return new CompositeHealthChecker(Arrays.asList(keystoreHealthChecker, truststoreHealthChecker));
    }

    @Bean
    @Qualifier("keystoreHealthChecker")
    public HealthChecker keystoreHealthChecker(@Autowired @Qualifier("keystoreCredentialVault") CredentialVault keystoreCredentialVault, @Value("${uuid2cpr.proxy.signature.keystore.alias}") String keystoreAlias) {
        return new CertificateHealthChecker(keystoreAlias, keystoreCredentialVault, "uuid2cprKeystore");
    }

    @Bean
    @Qualifier("keystoreCredentialVault")
    public CredentialVault keystoreCredentialVault(@Value("${uuid2cpr.proxy.signature.keystore.path}") String keystorePath, @Value("${uuid2cpr.proxy.signature.password}") String keystorePassword) {
        return new FileBasedCredentialVault(new Properties(), new File(keystorePath), keystorePassword);
    }

    @Bean
    @Qualifier("truststoreHealthChecker")
    public HealthChecker truststoreHealthChecker(@Autowired @Qualifier("truststoreCredentialVault") CredentialVault credentialVault, @Value("${uuid2cpr.proxy.encryption.keystore.alias}") String keystoreAlias) {
        return new CertificateHealthChecker(keystoreAlias, credentialVault, "uuid2cprTruststore");
    }

    @Bean
    @Qualifier("truststoreCredentialVault")
    public CredentialVault truststoreCredentialVault(@Value("${uuid2cpr.proxy.encryption.keystore.path}") String truststorePath, @Value("${uuid2cpr.proxy.encryption.password}") String truststorePassword) {
        return new FileBasedCredentialVault(new Properties(), new File(truststorePath), truststorePassword);
    }

    @Bean
    public SLALogger slaLogger(@Value("${app.name}") String appName, @Value("${app.short.name}") String shortAppName) {
        SLALogConfig slaLogConfig = new SLALogConfig(appName, shortAppName);
        return slaLogConfig.getSLALogger();
    }

    @Bean
    public Uuid2CprClient uuid2CprClient(@Autowired ObjectPool<LookupService> clientPool) {
        return new Uuid2CprClient(clientPool);
    }

    @Bean
    public ObjectPool<LookupService> clientPool(@Autowired PoolableObjectFactory<LookupService> clientFactory, @Value("${uuid2cpr.proxy.lookup.client.pool.size}") Integer clientPoolSize) {
        return new GenericObjectPool<>(clientFactory, clientPoolSize);
    }

    @Bean
    public PoolableObjectFactory<LookupService> clientFactory(@Autowired Uuid2CprProperties properties) {
        return new LookupServiceObjectFactory(properties);
    }

    @Bean
    public Uuid2CprProperties uuid2CprProperties(
            @Value("${uuid2cpr.proxy.signature.username}") String signatureUsername,
            @Value("${uuid2cpr.proxy.signature.password}") String signaturePassword,
            @Value("${uuid2cpr.proxy.signature.properties.file}") String signaturePropertiesFile,
            @Value("${uuid2cpr.proxy.encryption.username}") String encryptionUsername,
            @Value("${uuid2cpr.proxy.encryption.properties.file}") String encryptionPropertiesFile,
            @Value("${uuid2cpr.proxy.applies.to}") String appliesTo,
            @Value("${uuid2cpr.proxy.security.token.service.url}") String securityTokenServiceUrl,
            @Value("${uuid2cpr.proxy.lookup.service.url}") String lookupServiceUrl
    ) {
        Uuid2CprProperties properties = new Uuid2CprProperties();

        properties.setSignatureUsername(signatureUsername);
        properties.setSignaturePassword(signaturePassword);
        properties.setSignaturePropertiesFile(signaturePropertiesFile);
        properties.setEncryptionUsername(encryptionUsername);
        properties.setEncryptionPropertiesFile(encryptionPropertiesFile);
        properties.setAppliesTo(appliesTo);
        properties.setSecurityTokenServiceUrl(securityTokenServiceUrl);
        properties.setLookupServiceUrl(lookupServiceUrl);

        return properties;
    }

	@Bean
	public static PropertySourcesPlaceholderConfigurer propertySourcesPlaceholderConfigurer() {
		return new PropertySourcesPlaceholderConfigurer();
	}
}
