/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2018 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
package dk.nsp.uuid2cpr.proxy.ws;

import dk.nsp.uuid2cpr.proxy.health.HealthChecker;
import dk.nsp.uuid2cpr.proxy.health.HealthStatus;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriter;
import org.apache.commons.lang3.CharSet;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.context.support.SpringBeanAutowiringSupport;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

@WebServlet({"/*"})
public class LookupServlet extends HttpServlet {
    @Value("${git.commit.id}")
    private String version;

    @Autowired
	private OpenApiWriter openApiWriter;

    @Autowired
    private HealthChecker healthChecker;

    public LookupServlet() {

    }

    public LookupServlet(String version, OpenApiWriter openApiWriter, HealthChecker healthChecker) {
        this.version = version;
        this.openApiWriter = openApiWriter;
        this.healthChecker = healthChecker;
    }

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        SpringBeanAutowiringSupport.processInjectionBasedOnServletContext(this, config.getServletContext());
    }

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String pathInfo = request.getPathInfo();
		if(pathInfo.endsWith("/openapi")) {
            writeOpenApiResponse(response);
        }
        else if(pathInfo.endsWith("/status")) {
            writeStatusResponse(response);
        }
        else if(pathInfo.endsWith("/version")) {
            writeVersionResponse(request, response);
        }
		else {
            writeNotFoundResponse(response);
        }
	}

    private void writeOpenApiResponse(HttpServletResponse response) throws IOException {
        openApiWriter.writeOpenApi(response);
    }

    private void writeStatusResponse(HttpServletResponse resp) throws IOException {
        HealthStatus healthStatus = healthChecker.checkHealth();
        if(healthStatus.isHealthy()) {
            writeOkResponse(String.join("\n", healthStatus.getMessages()), resp);
        }
        else {
            writeErrorResponse(String.join("\n", healthStatus.getMessages()), resp);
        }
    }

    private void writeVersionResponse(HttpServletRequest request, HttpServletResponse response) throws IOException {
        if(version != null && !version.isEmpty()) {
            writeOkResponse(version, response);
        }
        else {
            writeErrorResponse("Could not determine version.", response);
        }
    }

    private void writeNotFoundResponse(HttpServletResponse response) {
        response.setStatus(HttpServletResponse.SC_NOT_FOUND);
    }

    private void writeOkResponse(String contents, HttpServletResponse response) throws IOException {
        writeResponse(contents, HttpStatus.SC_OK, response);
    }

    private void writeErrorResponse(String contents, HttpServletResponse response) throws IOException {
        writeResponse(contents, HttpStatus.SC_INTERNAL_SERVER_ERROR, response);
    }

    private void writeResponse(String contents, int status, HttpServletResponse response) throws IOException {
        response.setContentType("text/plain; charset=utf-8");
        response.setStatus(status);

        if(contents != null) {
            PrintWriter out = response.getWriter();
            try {
                out.println(contents);
            }
            finally {
                out.flush();
            }
        }
    }
}
