package dk.nsp.uuid2cpr.proxy.health;

import dk.sosi.seal.vault.CredentialVault;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.cert.Certificate;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class CertificateHealthCheckerTest {
    private CertificateHealthChecker subject;

    private static final String VAULT_NAME = "vault";

    @Test(expected = IllegalStateException.class)
    public void checkHealth_misconfiguredVault_throwsException() throws Exception {
        // Arrange
        String alias = "alias";
        CredentialVault vault = buildEmptyVault();

        subject = new CertificateHealthChecker(alias, vault, VAULT_NAME);

        // Act
        HealthStatus result = subject.checkHealth();

        // Assert
    }

    @Test(expected = IllegalStateException.class)
    public void checkHealth_wrongTypeOfCertificate_throwsException() throws Exception {
        // Arrange
        Certificate certificate = Mockito.mock(Certificate.class);
        String alias = "alias";
        CredentialVault vault = buildVaultWithCertificate(certificate, alias);

        subject = new CertificateHealthChecker(alias, vault, VAULT_NAME);

        // Act
        HealthStatus result = subject.checkHealth();

        // Assert
    }

    @Test
    public void checkHealth_certificateExpired_unhealthy() throws Exception {
        // Arrange
        String alias = "alias";
        Certificate certificate = buildExpiredCertificate();
        CredentialVault vault = buildVaultWithCertificate(certificate, alias);

        // Act
        subject = new CertificateHealthChecker(alias, vault, VAULT_NAME);
        HealthStatus result = subject.checkHealth();

        // Assert
        assertFalse(result.isHealthy());
        assertTrue(result.getMessages().stream().anyMatch(m -> m.contains("expired")));
    }

    @Test
    public void checkHealth_certificateNotYetValid_unhealthy() throws Exception {
        // Arrange
        String alias = "alias";
        Certificate certificate = buildNotYetValidCertificate();
        CredentialVault vault = buildVaultWithCertificate(certificate, alias);

        // Act
        subject = new CertificateHealthChecker(alias, vault, VAULT_NAME);
        HealthStatus result = subject.checkHealth();

        // Assert
        assertFalse(result.isHealthy());
        assertTrue(result.getMessages().stream().anyMatch(m -> m.contains("valid yet")));
    }

    @Test
    public void checkHealth_certificateValid_ok() throws Exception {
        // Arrange
        String alias = "alias";
        Certificate certificate = buildValidCertificate();
        CredentialVault vault = buildVaultWithCertificate(certificate, alias);

        // Act
        subject = new CertificateHealthChecker(alias, vault, VAULT_NAME);
        HealthStatus result = subject.checkHealth();

        // Assert
        assertTrue(result.isHealthy());
    }

    private CredentialVault buildEmptyVault() throws Exception {
        CredentialVault vault = Mockito.mock(CredentialVault.class);

        KeyStore keyStore = Mockito.mock(KeyStore.class);
        Mockito.when(keyStore.getCertificate(Mockito.anyString())).thenThrow(KeyStoreException.class);

        Mockito.when(vault.getKeyStore()).thenReturn(keyStore);

        return vault;
    }

    private CredentialVault buildVaultWithCertificate(Certificate certificate, String alias) throws Exception {
        CredentialVault vault = Mockito.mock(CredentialVault.class);

        KeyStore keyStore = Mockito.mock(KeyStore.class);
        Mockito.when(keyStore.getCertificate(alias)).thenReturn(certificate);

        Mockito.when(vault.getKeyStore()).thenReturn(keyStore);

        return vault;
    }

    private X509Certificate buildExpiredCertificate() throws Exception {
        X509Certificate certificate = Mockito.mock(X509Certificate.class);

        Mockito.doThrow(CertificateExpiredException.class).when(certificate).checkValidity();

        return certificate;
    }

    private X509Certificate buildNotYetValidCertificate() throws Exception {
        X509Certificate certificate = Mockito.mock(X509Certificate.class);

        Mockito.doThrow(CertificateNotYetValidException.class).when(certificate).checkValidity();

        return certificate;
    }

    private X509Certificate buildValidCertificate() {
        return Mockito.mock(X509Certificate.class);
    }
}