package dk.nsp.uuid2cpr.proxy.ws;

import dk.nsp.uuid2cpr.proxy.health.HealthChecker;
import dk.nsp.uuid2cpr.proxy.health.HealthStatus;
import dk.nsp.uuid2cpr.proxy.service.OpenApiWriter;
import org.apache.http.HttpStatus;
import org.jboss.resteasy.mock.MockHttpRequest;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class LookupServletTest {
    private LookupServlet subject;

    @Mock
    private OpenApiWriter openApiWriter;

    @Mock
    private HealthChecker healthChecker;

    private static final String VERSION = "abcdef";

    @Before
    public void setup() {
        subject = new LookupServlet(VERSION, openApiWriter, healthChecker);
    }

    @Test
    public void doGet_openapi_returnsDescriptor() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/openapi");

        // Act
        subject.doGet(request, response);

        // Assert
        Mockito.verify(openApiWriter).writeOpenApi(response);
    }

    @Test
    public void doGet_status_returns200WhenHealthy() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/status");

        String description = "good";
        Mockito.when(healthChecker.checkHealth()).thenReturn(HealthStatus.ok(description));

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(HttpStatus.SC_OK, response.getStatus());
        assertTrue(response.getContentAsString().contains(description));
    }

    @Test
    public void doGet_status_returns500WhenUnhealthy() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/status");

        String description = "not good";
        Mockito.when(healthChecker.checkHealth()).thenReturn(HealthStatus.unhealthy(description));

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(HttpStatus.SC_INTERNAL_SERVER_ERROR, response.getStatus());
        assertTrue(response.getContentAsString().contains(description));
    }

    @Test
    public void doGet_version_returnsVersion() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/version");

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(200, response.getStatus());
        assertTrue(response.getContentAsString().contains(VERSION));
    }

    @Test
    public void doGet_detectsMissingVersion() throws Exception {
        // Arrange
        subject = new LookupServlet(null, openApiWriter, healthChecker);

        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/version");

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(500, response.getStatus());
    }

    @Test
    public void doGet_unknownPath_returns404() throws Exception {
        // Arrange
        MockHttpServletRequest request = new MockHttpServletRequest();
        MockHttpServletResponse response = new MockHttpServletResponse();

        request.setPathInfo("/bla");

        // Act
        subject.doGet(request, response);

        // Assert
        assertEquals(HttpStatus.SC_NOT_FOUND, response.getStatus());
    }
}